from scopus.tests.test_CitationOverview import *
from scopus.tests.test_ScopusAbstract import *
from scopus.tests.test_ScopusAffiliation import *
from scopus.tests.test_ScopusAuthor import *
from scopus.tests.test_ScopusSearch import *
from scopus.tests.test_AuthorSearch import *
